/*****************************************************************************
 * File:			alarm.c
 * Description:		Alarm application file
 * Project:			Intrusion system and access control
 * 					(Sistema de intrusión y control de accesos)
 *
 * Version:			1.0		02/01/2017
 * Author:			Daniel Gómez
 *****************************************************************************/


#include "alarm.h"
#include "log.h"
#include "gpio.h"
#include "wifly.h"


	/* Status variables*/
int alarmStatus = OFF;
int magnethicStatus = OFF;
int volumethricStatus = OFF;


	/* Initializes pins and interruptions */
void alarmInit(){
		Gpio_config(1,GPIO_IN); 	//Initializes magnethic contact pin
		Gpio_config(2,GPIO_IN);		//Initializes volumethric detector pin
		Gpio_config(5,GPIO_IN);		//Initializes armed/disarmed switch pin
		Gpio_config(0,GPIO_OUT);	//Initializes green status LED pin
		Gpio_config(6,GPIO_OUT);	//Initializes red status LED pin
		Gpio_config(7,GPIO_OUT);	//Initializes buzzer pin
		Gpio_config(8,GPIO_OUT);	//Initializes spotlight pin
		Gpio_config(9,GPIO_OUT);	//Initializes electric locker pin
		LPC_PINCON->PINMODE_OD0 |= (1<<6);	//open drain mode for status LED
		LPC_PINCON->PINMODE_OD0 |= (1);		//open drain mode for status LED
		LPC_PINCON->PINSEL4 &= ~(3<<24); 	//puts '0' in bits of P2.12
		LPC_PINCON->PINSEL4 |= (1<<24);		//P2.12 as EINT2
		LPC_GPIOINT->IO2IntEnF |= (1<<12);  //P2.12 as falling edge
		LPC_SC->EXTMODE |= 1<<2;      		//P2.12 as edge interruption
		LPC_SC->EXTPOLAR &= ~(1<<2);       	//P2.12 as falling polarity
		LPC_SC->EXTINT |=1<<2;				//resets EXTINT bits for P2.12
		NVIC_EnableIRQ(EINT2_IRQn);			//enables EINT2 in NVIC
}

	/* Activates the alarm */
void alarmOn(int typeAlarm ) {

	switch (typeAlarm) {
			case MAGNETHIC:
				if(alarmStatus==OFF){
					Log_print("Alarma Activada\r");
					Log_print("Detección Magnético\r");
					alarmStatus=ON;
					magnethicStatus=ON;
					volumethricStatus=OFF;
					Gpio_set(7,1);
					Gpio_set(8,1);
					wiflyEmail("dungeon713@hotmail.com", "ALARMA", "Se ha producido una intrusión por contacto magnético");
					Log_print("E-mail de alarma enviado\r");
				}else{
					if(volumethricStatus==ON){
						Log_print("Detección Magnético\r");
						volumethricStatus=OFF;
					}

				}
				break;

			case VOLUMETHRIC:
				if(alarmStatus==OFF){
					Log_print("Alarma Activada\r");
					Log_print("Detección Volumétrico\r");
					alarmStatus=ON;
					magnethicStatus=OFF;
					volumethricStatus=ON;
					Gpio_set(7,1);
					Gpio_set(8,1);
					wiflyEmail("dungeon713@hotmail.com", "ALARMA", "Se ha producido una intrusión por volumétrico");
					Log_print("E-mail de alarma enviado\r");
				}else{
					if(magnethicStatus==ON){
						Log_print("Detección Volumétrico\r");
						magnethicStatus=OFF;
					}

				}
				break;

			case BOTH:
				if(((magnethicStatus==OFF)&&(volumethricStatus==ON))||((magnethicStatus==ON)&&(volumethricStatus==OFF))){
					magnethicStatus=OFF;
					volumethricStatus=OFF;
				}

				if(alarmStatus==OFF){
					Log_print("Alarma Activada\r");
					Log_print("Detección Magnético y Volumétrico\r");
					alarmStatus=ON;
					magnethicStatus=OFF;
					volumethricStatus=OFF;
					Gpio_set(7,1);
					Gpio_set(8,1);
				}else{
					if((magnethicStatus==OFF)&&(volumethricStatus==OFF)){
						Log_print("Detección Magnético y Volumétrico\r");
						magnethicStatus=ON;
						volumethricStatus=ON;
					}
				}
				break;

			default:
				;
				break;
		}
}

	/* Deactivates the alarm */
void alarmOff() {

	if(alarmStatus==ON){
		Log_print("Alarma restablecida\r");
		alarmStatus=OFF;
		Gpio_set(7,0);
		Gpio_set(8,0);
		wiflyEmail("dungeon713@hotmail.com", "ALARMA RESTABLECIDA", "Se ha restablecido la alarma");
		Log_print("E-mail de restableciemiento de alarma enviado\r");
	}else{
		Gpio_set(7,0);
		Gpio_set(8,0);
	}
}

	/* Get method for the alarm status */
uint8_t alarm_getStatus(){

	if (alarmStatus==ON)
	{
		return 1;
	}
	else{
		return 0;
	}
}
