/*****************************************************************************
 * File:			mifare.c
 * Description:		Reader application file
 * Project:			Intrusion system and access control
 * 					(Sistema de intrusión y control de accesos)
 *
 * Version:			1.0		02/01/2017
 * Author:			Daniel Gómez
 *****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include "FreeRTOS.h"
#include "task.h"
#include "LPC17xx.h"
#include "type.h"
#include "gpio.h"
#include "serial.h"
#include "wifly.h"
#include "log.h"



#define MAX_BITS 100                 // max number of bits

static unsigned long databits[MAX_BITS];    // stores all of the data bits
static unsigned int bitCount=0;              // number of bits currently captured

char buffTx[40];
uint32_t cardCode=0;

int DataMifare0=0;							//controls if there is a '0' bit to store
int DataMifare1=0;							//controls if there is a '1' bit to store
int accepted=0;								//controls if there is an accepted card

int intrusionArmed=1;						//stores the status of the intrussion system

void checkID();								//checks the card ID


	/* Initizlizes pins and interruptions */
void mifareInit()
{
	LPC_PINCON->PINSEL4 &= ~(15<<20); 	//puts '0' in bits of P2.10 and P2.11
	LPC_PINCON->PINSEL4 |= (5<<20);		//P2.10 as EINT0 and P2.11 as EINT1
	LPC_GPIOINT->IO2IntEnF |= (3<<10);  //P2.10 and P2.11 as falling edge
	LPC_SC->EXTMODE |= 3;      			//P2.10 and P2.11 as edge interruption
	LPC_SC->EXTPOLAR &= ~(3);       	//P2.10 and P2.11 as falling polarity
	LPC_SC->EXTINT |= 3;				//resets EXTINT bits for P2.10 and P2.11
	NVIC_EnableIRQ(EINT0_IRQn);			//enables EINT0 in NVIC
	NVIC_EnableIRQ(EINT1_IRQn);			//enables EINT1 in NVIC
}

	/* puts 1 in the variable */
void data0() {
	DataMifare0=1;
}

	/* puts 1 in the variable */
void data1() {
	DataMifare1=1;
}

/* puts 1 in the variable */
void Armed() {
	intrusionArmed=1;
}

/* puts 0 in the variable */
void Disarmed() {
	intrusionArmed=0;
}

	/* store bits of interruptions in order to catch the 26 bits the reader sends */
void cardReader(){

	if(DataMifare0==1)
	{
		bitCount++;
		DataMifare0=0;
		if(bitCount==26)
		{
			unsigned char i;
			for (i=0; i<26; i++)
			{
				cardCode <<=1;
				cardCode |= databits[i];
			}
			checkID();
		}
	}
	else{
		if(DataMifare1==1)
		{
			databits[bitCount] = 1;
			bitCount++;
			DataMifare1=0;
			if(bitCount==26)
			{
				unsigned char i;
				for (i=0; i<26; i++)
				{
					cardCode <<=1;
					cardCode |= databits[i];
				}
				checkID();
			}
		}
	}
}

	/* check if the card identification is in the data base */
void checkID(){
	if((cardCode==5019868)||(cardCode==38702652)||(cardCode==5164765)){
		Log_print("ACCEPTED ID\r");
		accepted=1;
		sprintf(buffTx,"ID: %d\r", cardCode);
		Log_print(buffTx);
		bitCount=0;
		unsigned char i;
		for (i=0; i<MAX_BITS; i++)
		{
			databits[i] = 0;
		}
	}else{
		Log_print("DENIED ID\r");
	sprintf(buffTx,"ID: %d\r", cardCode);
	Log_print(buffTx);
		accepted=0;
		bitCount=0;
		cardCode=0;
		unsigned char i;
		for (i=0; i<MAX_BITS; i++)
		{
			databits[i] = 0;
		}
	}
}

	/* Get function for accepted or denied card */
uint8_t acceptedID(){

	if(accepted==1)
	{
		accepted=0;
		return 1;
	}
	else{
		return 0;
	}
}


	/* Get function for the cardcode number */
uint32_t getCardCode(void){

	return (cardCode);
}

	/* Resets cardCode number and bitCount */
void resetCardcode(){
	cardCode=0;
	bitCount=0;
}
