/*****************************************************************************
 * File:			gpio.h
 * Description:		Driver for General Purpose Input/Output ports on LPC1769
 *
 * Author:			Daniel Gómez
 * Created:			02/01/2017
 *****************************************************************************/

#ifndef GPIO_H_
#define GPIO_H_

#include "LPC17xx.h"
#include "stdint.h"

#define GPIO_LOW 0
#define GPIO_HIGH 1
#define GPIO_INTERRUPT_FE 0
#define GPIO_INTERRUPT_RE 1

typedef enum gpio_mode_operations{
	GPIO_OUT,
	GPIO_IN,
} gpio_mode_operation;


typedef enum gipo_errors{
	GPIO_NON_ERROR,
	GPIO_ERROR_NOT_SUPPORTED
} gipo_error;


/**
 * Config pad (defined by Pport.pin) to speific mode
 *
 * @param pad	Pad identifier. Pad is port*32 + bit, e.g.: pad 35, is port 1 bit 3, pad 17 is port 0 bit 17, ... (only supported 0 and 1)
 * @param mode	Mode operation (GPIO_OUT, GPIO_IN, ...)
 */
gipo_error Gpio_config(uint8_t pad, gpio_mode_operation mode);


/**
 * Get status of pad (defined by Pport.pin)
 *
 * @param pad	Pad identifier. Pad is port*32 + bit, e.g.: pad 35, is port 1 bit 3, pad 17 is port 0 bit 17, ... (only supported 0 and 1)
 *
 */
uint8_t GPIO_getValue(uint8_t pad);


/**
 * Set value val in GPIO.
 *
 * @param pad 	pad number
 * @param val	GPIO_LOW or GPIO_HIGH
 */
void Gpio_set(uint8_t pad, uint8_t val);

/**
 * Configure interrupts in GPIO (EINT3).
 *
 * @param pad	Pad identifier. Pad is port*32 + bit, e.g.: pad 35, is port 1 bit 3, pad 17 is port 0 bit 17, ... (only supported 0 and 1)
 * @param mode	Mode operation (GPIO_OUT, GPIO_IN, ...)
 */
void GPIOSetInterrupt ( uint32_t pad, uint32_t mode );



#endif /* GPIO_H_ */
