/*****************************************************************************
 * File:			gpio.c
 * Description:		Driver for General Purpose Input/Output ports on LPC1769
 *
 * Author:			Daniel Gómez
 * Created:			02/01/2017
 *****************************************************************************/


#include "gpio.h"
#include "log.h"
#include "assertlib.h"
#include "type.h"


#define GPIO_PORT0	0
#define GPIO_PORT1	1		// NOT SUPPORTED YET
#define GPIO_PORT2	2		// NOT SUPPORTED YET
#define GPIO_PORT3	3		// NOT SUPPORTED YET


gipo_error Gpio_config(uint8_t pad, gpio_mode_operation mode ) {

	uint8_t port;
	uint8_t pin;

	port = pad / 32;
	pin = pad % 32;

	if (mode == GPIO_IN) {
		switch(port){
					// It needs to modify 2 bits to config 1 pin ==> pin 0: bit 1 and 0, pin 1: bits 3 and 2, ...
					case GPIO_PORT0: {
						if (pin <= 15) {		// controled by register 0

							LPC_PINCON->PINSEL0		&= ~((3)<<(pin<<1));
							LPC_PINCON->PINMODE0	|= ((3)<<(pin<<1));
							LPC_GPIO0->FIODIR 		&= ~(1<<pin);
						} else if (pin > 15){	// controled by register 1
							LPC_PINCON->PINSEL1	&= ~((3)<<(pin<<1));
							LPC_PINCON->PINMODE1	|= ((3)<<(pin<<1));
							LPC_GPIO0->FIODIR 		&= ~(1<<pin);
						}else{
							return 0;
						}

						break;
					}

					case 1: {
						if(pin <= 15){		// controled by register 2
							LPC_PINCON->PINSEL2		&= ~((3)<<(pin<<1));
							LPC_PINCON->PINMODE2	|= ((3)<<(pin<<1));
							LPC_GPIO1->FIODIR 		&= ~(1<<pin);
						}else if(pin > 15){	// controled by register 3
							LPC_PINCON->PINSEL3	&= ~((3)<<(pin<<1));
							LPC_PINCON->PINMODE3	|= ((3)<<(pin<<1));
							LPC_GPIO1->FIODIR 		&= ~(1<<pin);
						}else{
							return 0;
						}

						break;
					}
					case 2: {
						if(pin <= 15){		// controled by register 2
							LPC_PINCON->PINSEL4		&= ~((3)<<(pin<<1));
							LPC_PINCON->PINMODE4	|= ((3)<<(pin<<1));
							LPC_GPIO1->FIODIR 		&= ~(1<<pin);
						}else if(pin > 15){	// controled by register 3
							LPC_PINCON->PINSEL3	&= ~((3)<<(pin<<1));
							LPC_PINCON->PINMODE3	|= ((3)<<(pin<<1));
							LPC_GPIO1->FIODIR 		&= ~(1<<pin);
						}else{
							return 0;
						}

						break;
						}

					default:
						assert(0);
						return GPIO_ERROR_NOT_SUPPORTED;

			}
	}
	else if(mode==GPIO_OUT){


	switch(port){
			// It needs to modify 2 bits to config 1 pin ==> pin 0: bit 1 and 0, pin 1: bits 3 and 2, ...
			case GPIO_PORT0: {
				if (pin <= 15) {		// controled by register 0

					LPC_PINCON->PINSEL0		&= ~((3)<<(pin<<1));
					//LPC_PINCON->PINMODE0	|= (3)<<(pin<<1);
					LPC_GPIO0->FIODIR 		|= (1<<pin);
				} else if (pin > 15){	// controled by register 1
					LPC_PINCON->PINSEL1	&= ~((3)<<(pin<<1));
					//LPC_PINCON->PINMODE1	|= ((3)<<(pin<<1));
					LPC_GPIO0->FIODIR 		|= (1<<pin);
				}else{
					return 0;
				}

				break;
			}

			case 1: {
				if(pin <= 15){		// controled by register 2
					LPC_PINCON->PINSEL2		&= ~((3)<<(pin<<1));
					//LPC_PINCON->PINMODE2	&= ~((3)<<(pin<<1));
					LPC_GPIO1->FIODIR 		|= (1<<pin);
				}else if(pin > 15){	// controled by register 3
					LPC_PINCON->PINSEL3	&= ~((3)<<(pin<<1));
					//LPC_PINCON->PINMODE3	&= ~((3)<<(pin<<1));
					LPC_GPIO1->FIODIR 		|= (1<<pin);
				}else{
					return 0;
				}

				break;
			}


			case 2: {
				if(pin <= 15){		// controled by register 4
					LPC_PINCON->PINSEL4		&= ~((3)<<(pin<<1));
					LPC_PINCON->PINMODE4	|= ((3)<<(pin<<1));
					LPC_GPIO2->FIODIR 		|= (1<<pin);
				}else if(pin > 15){	// controled by register 5
					LPC_PINCON->PINSEL5	&= ~((3)<<(pin<<1));
					LPC_PINCON->PINMODE5	|= ((3)<<(pin<<1));
					LPC_GPIO2->FIODIR 		|= (1<<pin);
				}else{
					return 0;
				}

				break;
			}



			default:
				assert(0);
				return GPIO_ERROR_NOT_SUPPORTED;

	}}

	return GPIO_NON_ERROR;
}

uint8_t GPIO_getValue(uint8_t pad){

	uint8_t port;
	uint8_t pin;

	port = pad / 32;
	pin = pad % 32;

	switch(port){

			case GPIO_PORT0:
				if (pin <= 31) {
					if((LPC_GPIO0->FIOPIN)&(1<<pin)){
						return 1;
					}
					else{
						return 0;
					}
				}else{
					assert(0);
				}
				break;

			case GPIO_PORT1:
							if (pin <= 31) {
								if(LPC_GPIO1->FIOPIN&(1<<pin)){
									return 1;
								}
								else{
									return 0;
								}
							}else{
								assert(0);
							}
							break;

			default:
				;
				assert(0);
				break;

	}
	assert(0);
	return TRUE;
}

void Gpio_set(uint8_t pad, uint8_t val) {
	uint8_t port;
	uint8_t pin;

	port = pad / 32;
	pin = pad % 32;

	switch (port) {
			case GPIO_PORT0:
				if(pin <= 31){
					if (GPIO_LOW == val) {
						LPC_GPIO0->FIOCLR = 1 << pin;		// put 0
					}else{
						LPC_GPIO0->FIOSET = 1 << pin;		// put 1
					}

				} else {
					assert(0);
				}
				break;

			case GPIO_PORT1:
				if(pin <= 31){
					if (GPIO_LOW == val) {
						LPC_GPIO1->FIOCLR = 1 << pin;		// put 0
					}else{
						LPC_GPIO1->FIOSET = 1 << pin;		// put 1
					}

				} else {
					assert(0);
				}
				break;

			default:
				;
				assert(0);
				break;
		}

}
void GPIOSetInterrupt ( uint32_t pad, uint32_t mode )
{
		uint8_t port;
		uint8_t pin;

		port = pad / 32;
		pin = pad % 32;

		//  Dir es 0 para falling edge y 1 para rising edge interrupt, en teoria se podria establecer tambien para los dos
	if (port == 0)
	{
		if (mode == GPIO_INTERRUPT_FE)
		{
			LPC_GPIOINT->IO0IntEnF |= (1<<pin);
		}
		else if (mode == GPIO_INTERRUPT_RE)
		{
			LPC_GPIOINT->IO0IntEnR |= (1<<pin);
		}
	}
	else if (port == 2)
	{
		if (mode == GPIO_INTERRUPT_FE)
		{
			LPC_GPIOINT->IO2IntEnF |= (1<<pin);
		}
		else if (mode == GPIO_INTERRUPT_RE)
		{
			LPC_GPIOINT->IO2IntEnR |= (1<<pin);
		}
	}

	NVIC_EnableIRQ(EINT3_IRQn);
}



